#include "tester_main.h"


TESTER_ProjectTypeDef TESTER = {0};


/**s
  * @brief    Инициализация всех модулей для работы тестера
  */
void TESTER_Init(TESTER_ProjectTypeDef *tester)
{
  TESTER_InterfaceInit(&tester->SwStart, &tester->leds);
  GPIO_LED_Blink_Start(&tester->leds.LED1, 100);
  
  tester->delay = 250;
  tester->delay_en = 0;
  
  tester->func.disable_reset_call = 0;
  tester->htest = &hTestDiode;
  TESTER_HandleInit(tester->htest, &tester->leds);
  
  tester->hmodbus = &hmodbus1;
  tester->mbdata = &MB_DATA;
  MODBUS_FirstInit();
}


/**s
  * @brief    Функция подготовки тестера к while(1)
  */
void TESTER_pre_while(TESTER_ProjectTypeDef *tester)
{
  TESTER_Set_Default_Settings(tester);
  TESTER_UpdateSettings(tester->htest, tester->mbdata);
  
  RS_Receive_IT(tester->hmodbus, &MODBUS_MSG);
  HAL_TIM_Base_Start(&hmcstim);
  
  
  tester->f.flag_init_done = 1;
  GPIO_LED_Blink_Start(&tester->leds.LED1, LED_BLINK_AS_ON);
}



/**
  * @brief    Главная функция работы тестера
  */
void TESTER_main_while(TESTER_ProjectTypeDef *tester)
{
  if(tester->delay_en)
    msDelay(tester->delay);
  
  
  /* ПОДТЯГИВАНИЕ СОСТОЯНИЯ КЛЮЧЕЙ С МОДБАС */
  TESTER_TestOneSwitch(tester->htest->SwPhaseForward.SW_Port, tester->htest->SwPhaseForward.SwHI_Pin,
                        tester->mbdata->Coils.KeyForwardHiTest);
  TESTER_TestOneSwitch(tester->htest->SwPhaseForward.SW_Port, tester->htest->SwPhaseForward.SwLO_Pin,
                        tester->mbdata->Coils.KeyForwardLoTest);
  TESTER_TestOneSwitch(tester->htest->SwPhaseForward.SW_Port, tester->htest->SwPhaseReverse.SwHI_Pin,
                        tester->mbdata->Coils.KeyReverseHiTest);
  TESTER_TestOneSwitch(tester->htest->SwPhaseForward.SW_Port, tester->htest->SwPhaseReverse.SwLO_Pin,
                        tester->mbdata->Coils.KeyReverseLoTest);
  
  
  
  /* ТЕСТ В ОБРАТНОМ ВКЛЮЧЕНИИ */
  if(tester->func.test_diode_reverse)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_Reverse(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.ReversePeakVoltage = tester->htest->DiodeReversePeakVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_reverse = 0;
    
    tester->f.flag_test_active = 0;
    tester->mbdata->Coils.StartTest = 0;
  }
    
  /* ТЕСТ В ПРЯМОМ ВКЛЮЧЕНИИ */
  if(tester->func.test_diode_forward)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_Forward(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.ForwardVoltage = tester->htest->DiodeForwardVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_forward = 0;
    
    tester->f.flag_test_active = 0;
    tester->mbdata->Coils.StartTest = 0;
  }
  /* ТЕСТ ПЕРЕХОДА ИЗ ПРЯМОГО В ОБРАТНОЕ ВКЛЮЧЕНИЯ*/
  if(tester->func.test_diode_switch_connection)
  {
    tester->f.flag_test_active = 1;
    TESTER_TestDiode_SwitchConnection(tester->htest);
    
    /* Запись данных в modbus */
    tester->mbdata->InRegs.ReversePeakVoltage = tester->htest->DiodeReversePeakVolt*1000;
    tester->mbdata->InRegs.ForwardVoltage = tester->htest->DiodeForwardVolt*1000;
    
    if(tester->func.disable_reset_call == 0)
      tester->func.test_diode_switch_connection = 0;
    
    tester->f.flag_test_active = 0;
    tester->mbdata->Coils.StartTest = 0;
  }
}



/**
  * @brief    Функция работы интерфейса
  */
void TESTER_InterfaceHandle(TESTER_ProjectTypeDef *tester)
{
  if(tester->f.flag_init_done == 0)
    return;
  
  
  /* считывание режима тестера с модбас */
  tester->mode = *((TESTER_TestModeTypeDef *)&tester->mbdata->Coils) & 0x3;
  
  /* если кнопка нажата или пришла соответствующая комманда модбас */
  if(GPIO_Read_Swich(&tester->SwStart) || tester->mbdata->Coils.StartTest)
  {
    /* Обновление настроек тестера */
    TESTER_UpdateSettings(tester->htest, tester->mbdata);
    switch(tester->mode)
    {
      case TEST_FORWARD:
        tester->func.test_diode_forward = 1;
        break;
          
      case TEST_REVERSE:
        tester->func.test_diode_reverse = 1;
        break;
          
      case TEST_SWITCH_CONNECTION:
        tester->func.test_diode_switch_connection = 1;
        break;
      
      default:
        break;
    }
  }
  
  
  GPIO_LED_Blink_Handle(&tester->leds.LED1);
  
}


/**
  * @brief    Обновление настроек тестера
  */
void TESTER_UpdateSettings(TESTER_TestHandleTypeDef *htest, MB_DataStructureTypeDef *mbdata)
{
  htest->SwTimings.ticks_for_reverse.ticks = mbdata->HoldRegs.TimeForReverse;
  htest->SwTimings.ticks_deadtime.ticks = mbdata->HoldRegs.TimeDeadtime;
  htest->SwTimings.ticks_before_expected_peak.ticks = mbdata->HoldRegs.TimeBeforePeak;
  htest->SwTimings.ticks_before_test.ticks = mbdata->HoldRegs.TimeBeforeTest;  
  htest->SwTimings.ticks_for_forward.ticks = mbdata->HoldRegs.TimeForForward;
  
  htest->SwTimings.ticks_for_reverse.msdelay = mbdata->Coils.msTimeForReverse_enable;
  htest->SwTimings.ticks_deadtime.msdelay = mbdata->Coils.msTimeDeadtime_enable;
  htest->SwTimings.ticks_before_expected_peak.msdelay = mbdata->Coils.msTimeBeforePeak_enable;
  htest->SwTimings.ticks_before_test.msdelay = mbdata->Coils.msTimeBeforeTest_enable;  
  htest->SwTimings.ticks_before_test.msdelay = mbdata->Coils.msTimeForForward_enable;  
  
  TESTER_ADC_UpdateSettings(htest->adc, mbdata);
}


/**
  * @brief    Инициализация тестера по дефолтным настрйокам в tester_config.h
  */
void TESTER_Set_Default_Settings(TESTER_ProjectTypeDef *tester)
{
  tester->mbdata->Coils.ForwardTest = 1;
  tester->mbdata->Coils.ReverseTest = 1;
  
  /* Настройка пинов для обратного включения */
  tester->htest->SwPhaseForward.SW_Port   = SWITCH_PHASES_Port;
  tester->htest->SwPhaseForward.SwHI_Pin  = SWITCH_PHASE_A_POS_Pin;
  tester->htest->SwPhaseForward.SwLO_Pin  = SWITCH_PHASE_B_NEG_Pin;
  
  /* Настройка пинов для прямого включения */
  tester->htest->SwPhaseReverse.SW_Port   = SWITCH_PHASES_Port;
  tester->htest->SwPhaseReverse.SwHI_Pin  = SWITCH_PHASE_B_POS_Pin;
  tester->htest->SwPhaseReverse.SwLO_Pin  = SWITCH_PHASE_A_NEG_Pin;
  
  
  /* Настройка пинов для кнопки старта */
  tester->SwStart.Sw_Port           = SWITCH_START_Port;
  tester->SwStart.Sw_Pin            = SWITCH_START_Pin;
  tester->SwStart.Sw_FilterDelay    = SWITCH_ANTI_DREBEZG_DELAY;
  
  /* Настройка пинов для светодиода*/
  tester->leds.LED1.LED_Port        = LED1_Port;
  tester->leds.LED1.LED_Pin         = LED1_Pin;  
  
  /* Настройка таймингов по умолчанию для тестирования */
  tester->mbdata->HoldRegs.TimeBeforeTest             = DEF_TIME_BEFORE_TEST;
  tester->mbdata->Coils.msTimeBeforeTest_enable       = DEF_TIME_BEFORE_TEST_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeForReverse       = DEF_TIME_FOR_REVERSE;
  tester->mbdata->Coils.msTimeForReverse_enable = DEF_TIME_FOR_REVERSE_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeForForward             = DEF_TIME_FOR_FORWARD;
  tester->mbdata->Coils.msTimeForForward_enable       = DEF_TIME_FOR_FORWARD_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeDeadtime               = DEF_DEADTIME;
  tester->mbdata->Coils.msTimeDeadtime_enable         = DEF_DEADTIME_MS_DELAY;
  
  tester->mbdata->HoldRegs.TimeBeforePeak             = DEF_TIME_BEFORE_PEAK;
  tester->mbdata->Coils.msTimeBeforePeak_enable       = DEF_TIME_BEFORE_PEAK_MS_DELAY;
  
  
  /* Настройка АЦП */
  tester->mbdata->HoldRegs.Adc_PulseWidth             = TESTER_ADC_PULSE_EXPETCED_WIDTH;
  tester->mbdata->HoldRegs.Adc_CalibrValue            = ADC_VALUE_CALIBR;
  tester->mbdata->HoldRegs.Adc_ZeroValue              = ADC_VALUE_ZERO;
  tester->mbdata->HoldRegs.Adc_U_Calibr               = ADC_U_CALIBR;
}

/**
  * @brief    Задеркжка главного цикла
  */
void TESTER_main_delay(TESTER_ProjectTypeDef *tester)
{
  if(tester->delay_en)
    msDelay(tester->delay);
}